pub struct BlockRange { /* private fields */ }Expand description
A struct representing a block range in a CBQ file and stored in the Index
This is stored identically in memory and on disk.
Implementations