Error definitions
Enums
- Builder
Error - CbqError
- Error
- The main error type for the binseq library, encompassing all possible error cases that can occur during binary sequence operations.
- Extension
Error - Fastx
Encoding Error - Header
Error - Errors specific to processing and validating binary sequence headers
- Index
Error - Errors related to VBQ file indexing
- Read
Error - Errors that can occur while reading binary sequence data
- Write
Error - Errors that can occur while writing binary sequence data
Traits
- Into
Binseq Error - Trait for converting arbitrary errors into
Error